' SortIniReg: Sort .INI & .REG files (keeping comments lines)
' Arguments: <Input file> <Output File>
'
' Requirement: .Net should be installed.
'
' Notes:
'   Empty lines are related to the following block (Section or Key).
'   Lines starting with a space are related to the previous block (Section or Key).
'   Comment are related to following block except if the following line is an empty line or is starting with a space.
'
' Author: (c) 2007, Michel Dessaintes, michel.dessaintes(at)free.fr, http://congesrfv.free.fr
'
' History:
'   2007/04/03 v1.0 : Initial release to Beyond Compare, http://www.scootersoftware.com/
'   2007/04/21 v1.1 : Empty lines are related to the following block instead of the preceeding one.
'-------------------------------------------------------------
option explicit

' Requirments: .NET installed

Const ForReading = 1, ForWriting = 2, ForAppending = 3
Const TristateUseDefault = -2, TristateTrue = -1, TristateFalse = 0

Dim fso: Set fso = CreateObject("Scripting.FileSystemObject")

Dim sKeys() : Redim sKeys(1) : sKeys(0) = " " ' First key = file header
Dim sLines()
Dim iNrLines : iNrLines = 0
Dim sCurrentSection : sCurrentSection = " " ' Previous section is "File Header"
Dim sCurrentKey : sCurrentKey = " "         ' Previous Key is "File Header"
Dim bPreviousBlock : bPreviousBlock = True  ' True if current line is related to previous section or key (like in .reg files)

Dim sInputFile
Dim sOuputFile

Dim Args
Set Args = WScript.Arguments

If Args.count <> 2 Then
   WScript.Echo "Usage: [CScript | WScript] SortedIniMD.vbs <Input file> <Output file>"
   WScript.Quit 1
end If

sInputFile = Args.Item(0)
sOuputFile = Args.Item(1)

ReadINI sInputFile
SortINI
WriteINI sOuputFile

Function ReadINI(sInputFile)
    Dim sInputLine
    Dim sNames
    Dim fsInputFile:  Set fsInputFile = fso.OpenTextFile(sInputFile, ForReading, TristateUseDefault)

    Do While Not fsInputFile.AtEndOfStream
        sInputLine = fsInputFile.ReadLine()

        iNrLines = iNrLines+1
        Redim Preserve sKeys(iNrLines+1) :  sKeys(iNrLines) = MakeKey()
        Redim Preserve sLines(iNrLines+1) : sLines(iNrLines) = sInputLine

        If Len(sInputLine) = 0 Then                 ' Empty line
            ' This line is related to the following block.
            sKeys(iNrLines) = ""                    ' Will be filled on next block

        ElseIf Left(sInputLine & " ", 1) = " " Then ' Line starting with a space
            ' This line is related to the previous block.
            ' But if the previous line was a comment, this comment is related to previous block.
            If sKeys(iNrLines-1) = "" Then ReversePropagate()

        ElseIf Left(sInputLine, 1) = "[" Then       ' New Section
            ' Get the new section name
            sNames = Split(sInputLine, "]", -1, vbTextCompare)
            If IsArray(sNames) Then
                sCurrentSection = Mid(sNames(0), 2)
            Else
                MsgBox sInputLine, vbOKOnly, "Undelimited section name"
                sCurrentSection = "?" & sInputLine
            End If
            sCurrentKey = ""
            ReversePropagate()                      ' Set the sort keys of this line and the previous lines related to this section

        ElseIf Left(sInputLine, 1) = ";" Then       ' Comment for next block
            sKeys(iNrLines) = ""                    ' Will be filled on next block

        Else                                        ' Key=Value
            sNames = Split(sInputLine, "=", -1, vbTextCompare)
            If IsArray(sNames) Then
                sCurrentKey = sNames(0)
            Else
                MsgBox sInputLine, vbOKOnly, "Not value for Key"
                sCurrentKey = "?" & sInputLine
            End If
            ReversePropagate()                      ' Set the sort keys of the previous lines related to this key
        End If
    Loop

    fsInputFile.close
End Function

Function MakeKey()
    MakeKey = sCurrentSection & " " & sCurrentKey
End Function

Function ReversePropagate()
    ' Fill sKeys with the next block key starting from the last line
    Dim iLine
    Dim sKey : sKey = MakeKey()

    iLine = iNrLines
    Do
        sKeys(iLine) = sKey
        iLine = iLine - 1
    Loop While sKeys(iLine) = ""
End Function

Function WriteINI(sOutputFile)
    Dim iLine
    Dim fsOutStream : Set fsOutStream = fso.OpenTextFile(sOutputFile, ForWriting, TristateUseDefault)

    For iLine = 0 to iNrLines-1
        'fsOutStream.WriteLine(sKeys(iLine) & "=" & sLines(iLine))
        fsOutStream.WriteLine(sLines(iLine))
    Next
    fsOutStream.Close
End Function

Function SortINI()
    Dim iLine
    Dim sArray
    ' ArrayList requires .NET to be installed
    Dim oArrayList : Set oArrayList = CreateObject("System.Collections.ArrayList")
    Dim sSorted

    For iLine = 1 To iNrLines
        oArrayList.Add sKeys(iLine) & " " & Right("0000000000" & iLine, 10) & "=" & sLines(iLine)
    Next

    If False Then ' Debug
        Dim s
        Dim fsOut : Set fsOut = fso.OpenTextFile(sInputFile & ".Trace.txt", ForWriting, TristateUseDefault)
        For each s in oArrayList
            fsOut.WriteLine(s)
        Next
        fsOut.Close
        MsgBox sInputFile & ".Trace.txt" & vbNewLine & sOuputFile
    End If

    oArrayList.Sort
    set sSorted = oArrayList
    Set oArrayList = Nothing

    For iLine = 0 to iNrLines-1
        sArray = Split(sSorted(iLine), "=", -1, vbTextCompare)
        sLines(iLine) = Mid(sSorted(iLine), Len(sArray(0))+2)
        'Lines(iLine) = sSorted(iLine)
    Next
End Function
